@@ -65,11 +65,11 @@ function select_range_do_abook(){
             //    $select_range_hook = 1;
             //}
     
+            $function_bit = 'Abook';
+            $abook_func = 'FormAddrs' . $function_bit;
+            $result = select_range_jscript_function( $function_bit, $select_range_display_errors, 'abook' );
+
             if( select_range_display_test( count( $alist ) ) ) {
-                $function_bit = 'Abook';
-                $abook_func = 'FormAddrs' . $function_bit;
-        
-                $result = select_range_jscript_function( $function_bit, $select_range_display_errors, 'abook' );
                 // might as well add a "toggle all" link while we're at it
                 $result .= <<<EOF
 <script language="JavaScript" type="text/javascript">
@@ -97,8 +97,8 @@ EOF;
                     ) . 
                     "\n";
 
-                echo $result;
             }
+            echo $result;
         }
     }
 }
@@ -183,7 +183,8 @@ function select_range_do_print( $link_only = 0, $prefix = '' ){
     sqgetGlobalVar( 'SCRIPT_NAME', $script_name, SQ_SERVER );
 
     // kludgey attempt to disable this link on the search page
-    if( $javascript_on && $script_name != '/src/search.php' ){
+    // this should *really* be done a different way!
+    if( $javascript_on && ( substr( $script_name, -15 ) != '/src/search.php' ) ){
         global $numMessages, $startMessage, $PG_SHOWALL, $data_dir, $username;
         global $select_range_hook;
 
@@ -212,41 +213,42 @@ function select_range_do_print( $link_only = 0, $prefix = '' ){
             }
         }
 
-        if( select_range_display_test($message_count) ) {
-            global $mailbox;
-            $safe_name = preg_replace("/[^0-9A-Za-z_]/", '_', $mailbox);
-            $func_name = "SelectRange" . $safe_name;
-            $result = '';
-    
-            if( $link_only ){
+        $display_link = select_range_display_test($message_count);
+        global $mailbox;
+        $safe_name = preg_replace("/[^0-9A-Za-z_]/", '_', $mailbox);
+        $func_name = "SelectRange" . $safe_name;
+        $result = '';
+
+        if( $link_only ){
+            if( $display_link ){
                 $result .= $select_range_prefix . select_range_link( $func_name );
                 return( $result );
-                
-            } else {
-                // keep track of whether we already printed this function
-                // so we don't print it more than once
-                global $select_range_already_printed_jscript;
-        
-                if( ! isset( $select_range_already_printed_jscript ) ){
-                    $result .= 
-                        select_range_jscript_function( 
-                            $safe_name,
-                            $select_range_display_errors
-                        );
-                }
-        
-                if( !isset( $select_range_hook ) ){
-                    // need to put the link in a table so css rules are followed
-                    $result .= "\n<table>\n"
-                        . "<tr><td>\n"
-                        . select_range_link( $func_name )
-                        . "\n"
-                        . "</td></tr>\n"
-                        . "</table>\n";
-                }
-
-                echo ( $result );
             }
+            
+        } else {
+            // keep track of whether we already printed this function
+            // so we don't print it more than once
+            global $select_range_already_printed_jscript;
+    
+            if( ! isset( $select_range_already_printed_jscript ) ){
+                $result .= 
+                    select_range_jscript_function( 
+                        $safe_name,
+                        $select_range_display_errors
+                    );
+            }
+    
+            if( $display_link && !isset( $select_range_hook ) ){
+                // need to put the link in a table so css rules are followed
+                $result .= "\n<table>\n"
+                    . "<tr><td>\n"
+                    . select_range_link( $func_name )
+                    . "\n"
+                    . "</td></tr>\n"
+                    . "</table>\n";
+            }
+
+            echo ( $result );
         }
     }
 }
@@ -306,12 +308,31 @@ function select_range_jscript_function( $safe_name, $alerts = 1, $context = '' )
     $result = <<<EOF
 <script language="JavaScript" type="text/javascript">
 <!--
-function $func_name() { 
+function doSelect(element, e){
+  if( !e ) { 
+    return; 
+  }
+  var shifted = 0;
+  if( e.shiftKey || ( e.modifiers && ( e.modifiers & 4 ) ) ){ 
+    shifted = 1;
+  }
+  if( shifted ){
+    if( document.$form_name.elements[element].type == 'checkbox' ){
+      document.$form_name.elements[element].checked = true;
+    }
+    $func_name( true );
+  }
+}
+
+function $func_name( hide_alerts ) { 
+  var display_alert = $alerts;
+  if( hide_alerts ){
+    display_alert = false;
+  }
   var first_checked_message = -1;
   var last_checked_message = -1; 
   var unchecked_after_first = 0;
   var checked_after_unchecked = 0;
-  var display_alert = $alerts;
   for (var i = 0; i < document.$form_name.elements.length; i++) {
     if( document.$form_name.elements[i].type == 'checkbox' ) {
       if( document.$form_name.elements[i].checked ){
@@ -392,4 +413,27 @@ function select_range_validate_threshold( $option ){
 	 $option->new_value = $thresh;
     save_option( $option );
 }
+
+/**
+ * used for shift-click
+ * if we are called, we return an "onclick" javascript param
+ * which calls select range
+ */
+function select_range_do_checkbox_params($element_name,$always_show){
+    if( $always_show ){
+        return( " onclick=\"doSelect('$element_name',event);\" " );
+    }
+    else{
+        // kludgey attempt to only add "onclick" to checkboxes on addressbook page
+        // this should *really* be done a different way!
+        sqgetGlobalVar( 'SCRIPT_NAME', $script_name, SQ_SERVER );
+        if( substr( $script_name, -20 ) == '/src/addressbook.php' ){
+            return( " onclick=\"doSelect('$element_name',event);\" " );
+        }
+        else{
+            return( '' );
+        }
+    }
+}
+
 ?>
